/*IODEFv023 Signature
 * 
 * (C) Copyright IBM Corp. 1999-2001  All rights reserved.
 *
 * US Government Users Restricted Rights Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */

package testxmlsec;
import com.ibm.xml.dsig.AlgorithmFactory;
import com.ibm.xml.dsig.IDResolver;
import com.ibm.xml.dsig.KeyInfo;
import com.ibm.xml.dsig.SignatureContext;
import com.ibm.xml.dsig.XSignature;
import com.ibm.xml.dsig.XSignatureException;
import com.ibm.xml.enc.AlgorithmFactoryExtn;
import com.ibm.xml.enc.EncryptedKeyRetriever;
import com.ibm.xml.enc.KeyInfoResolver;
import com.ibm.xml.enc.StructureException;
import com.ibm.xml.enc.util.AdHocIdResolver;
import com.ibm.xml.enc.util.SameDocumentEncryptedKeyRetriever;


import java.security.Key;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.PrivateKey;
import java.security.UnrecoverableKeyException;
import java.security.cert.Certificate;

import org.apache.xpath.XPathAPI;
import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import xss4jsamples.enc.DOMCipher;
import xss4jsamples.enc.DOMCipher.Keyinfo;

import java.io.*;
import java.lang.reflect.*;
import java.security.PublicKey;
import java.security.cert.*;
import java.util.*;
import javax.xml.transform.TransformerException;

/** original code from
 * @version $Id: XMLSig.java,v 1.4 2002/12/26 10:49:21 imamu Exp $
 * @author Takeshi IMAMURA
 */
public class TestIODEF01xss4jdsig {
    //
    // Main
    //

    public static void main(String[] args) {
	if (args.length < 1) {
	    printUsage();
	    System.exit(1);
	}
	// J- 
// input document in arg[3]
	//	String File2sign="draft-ietf-inch-iodef-022.xml"
	File signedFile = new File("iodef02example1signed.xml");
	// J+
	String arg = args[0];
	if (arg.equals("-g") && args.length > 3) {
	    try {
		String s = null;
		if (!"*".equals(args[1])) {
		    s = args[1];
		}

		DOMCipher.Keyinfo ki = DOMCipher.getKeyinfo(args[2]);

		IDResolver ir = new AdHocIdResolver();

		Document d = DOMCipher.getDocument(args[3]);

		KeyStore ks = DOMCipher.getKeyStore(ki);

		SignatureContext sc = new SignatureContext();
		sc.setAlgorithmFactory(getAlgorithmFactory(s, ki, ir, d, ks));
		sc.setIDResolver(ir);

		for (int i = 4; i < args.length; i++) {
		    try {
			NodeList nl = XPathAPI.selectNodeList(d, args[i]);
			generateSignatures(nl, ki, ks, sc);

		    } catch (Exception e) {
			DOMCipher.printStackTrace(e);
		    }
		}
// print document to change to write to file signed.xml
//		DOMCipher.printDocument(d);

		printDocument(d, signedFile);

//		FileOutputStream d = new FileOutputStream (signedFile);
		
//		d.close();

// here is to use XML util to output to signedFile/fsigned
// from xmlsec CreateSignature:
// XMLUtils.outputDOMc14nWithCOmments (doc, fsigned)

	    } 
		catch (Exception e) 
		{
		DOMCipher.printStackTrace(e);
		System.exit(1);
	    }
	} else if (arg.equals("-v") && args.length > 3) {
	    try {
		String s = null;
		if (!"*".equals(args[1])) {
		    s = args[1];
		}

		DOMCipher.Keyinfo ki = DOMCipher.getKeyinfo(args[2]);

		IDResolver ir = new AdHocIdResolver();

		Document d = DOMCipher.getDocument(args[3]);

		KeyStore ks = DOMCipher.getKeyStore(ki);

		SignatureContext sc = new SignatureContext();
		sc.setAlgorithmFactory(getAlgorithmFactory(s, ki, ir, d, ks));
		sc.setIDResolver(ir);

		for (int i = 4; i < args.length; i++) {
		    try {
			arg = args[i];
			System.err.println(arg+":");
			NodeList nl = XPathAPI.selectNodeList(d, arg);
			validateSignatures(nl, ks, sc);

		    } catch (Exception e) {
			DOMCipher.printStackTrace(e);
		    }
		}
	    } catch (Exception e) {
		DOMCipher.printStackTrace(e);
		System.exit(1);
	    }
	} else if (arg.equals("-h")) {
	    printUsage();
	    System.exit(0);

	} else {
	    printUsage();
	    System.exit(1);
	}
    }
// Originally prints on stdout
// need to output to file
//		FileOutputStream f = new FileOutputStream (signedFile);
//		XMLUtils.outputDOMc14nWithComments(doc, f);
//		f.close();

	static void printDocument(Document doc, File signedFile) throws IOException 
	{
		//File sf = new File (signedFile);
		FileOutputStream f = new FileOutputStream (signedFile);
		OutputFormat of = new OutputFormat(doc);
		of.setPreserveSpace(true);
//		XMLSerializer xs = new XMLSerializer(System.out, of);
		XMLSerializer xs = new XMLSerializer(f, of);
		xs.serialize(doc);
//		System.out.println();
		f.close();
/*
		FileOutputStream f = new FileOutputStream ("signed.xml");
		f.setPreserveSpace(true);
		XMLSerializer xs = new XMLSerializer(System.out, f);
		f.close();
*/
	}

    private static void printUsage() {
	System.err.println("Usage: java enc.XMLSig <option> [<arg>...]");
	System.err.println("where <option> is:");
	System.err.println("   -g <provider> <keyinfo> <target> [<path>...]");
	System.err.println("        generate Signature elements");
	System.err.println("   -v <provider> <keyinfo> <target> [<path>...]");
	System.err.println("        validate Signature elements");
	System.err.println("   -h   print this message");
    }

    private static AlgorithmFactory getAlgorithmFactory(String provider,
							DOMCipher.Keyinfo keyinfo,
							IDResolver resolver,
							Document doc, KeyStore store)
	throws StructureException {

	AlgorithmFactoryExtn factory = new AlgorithmFactoryExtn();

	//IDResolver ir = new AdHocIdResolver();

	EncryptedKeyRetriever ekr = new SameDocumentEncryptedKeyRetriever(doc);

	KeyInfoResolver kir
	    = DOMCipher.getKeyInfoResolver(keyinfo, factory, resolver, ekr, store);
	kir.setOperationMode(KeyInfoResolver.DECRYPT_MODE);

	factory.setProvider(provider);
	factory.setKeyInfoResolver(kir);
	factory.setIdResolver(resolver);
	factory.setEncryptedKeyRetriever(ekr);

	return factory;
    }
	
	// Generate Signature 

    private static void generateSignatures(NodeList nodes, DOMCipher.Keyinfo keyinfo,
					   KeyStore store, SignatureContext context) {
	for (int i = 0, l = nodes.getLength(); i < l; i++) {
	    Node n = nodes.item(i);
	    if (n.getNodeType() == Node.ELEMENT_NODE) {
		Element el = (Element)n;
		if (isSignature(el)) {
		    try {
			generateSignature(el, keyinfo, store, context);

		    } catch (Exception ex) {
			DOMCipher.printStackTrace(ex);
		    }
		}
	    }
	}
    }
//Signature

    private static boolean isSignature(Element elem) {
	return (elem != null
		&& XSignature.XMLDSIG_NAMESPACE.equals(elem.getNamespaceURI())
		&& "Signature".equals(elem.getLocalName()));
    }

    private static void generateSignature(Element signature, DOMCipher.Keyinfo keyinfo,
					  KeyStore store, SignatureContext context)
	throws KeyStoreException, NoSuchAlgorithmException, UnrecoverableKeyException,
	       XSignatureException {
	Key k = getPrivateKey(signature, keyinfo, store);
	if (k == null) {
	    throw new NullPointerException("Key not specified or obtained");
	}
	context.sign(signature, k);
    }

// Input private key

    private static Key getPrivateKey(Element signature, DOMCipher.Keyinfo keyinfo,
				     KeyStore store)
	throws KeyStoreException, NoSuchAlgorithmException, UnrecoverableKeyException,
	       XSignatureException {
	Element e = KeyInfo.searchForKeyInfo(signature);
	Key key = null;
	if (e != null) {
	    String[] nn = new KeyInfo(e).getKeyNames();
	    if (nn != null) {
		for (int i = 0; key == null && i < nn.length; i++) {
		    String a = nn[i];
		    char[] p = (char[])keyinfo.fAlias2Keypass.get(a);
		    if (p != null) {
			Key k = store.getKey(a, p);
			if (k instanceof PrivateKey) {
			    key = k;
			}
		    }
		}
	    }
	}

	return key;
    }
// Validate Signature

    private static void validateSignatures(NodeList nodes, KeyStore store,
					   SignatureContext context) {
	for (int i = 0, l = nodes.getLength(); i < l; i++) {
	    Node n = nodes.item(i);
	    if (n.getNodeType() == Node.ELEMENT_NODE) {
		Element el = (Element)n;
		if (isSignature(el)) {
		    String s = "NG";
		    try {
			if (validateSignature(el, store, context)) {
			    s = "OK";
			}
		    } catch (Exception ex) {
			DOMCipher.printStackTrace(ex);
		    }
		    System.err.println(s);
		}
	    }
	}
    }

// Validate Signature

    private static boolean validateSignature(Element signature, KeyStore store,
					     SignatureContext context)
	throws KeyStoreException, XSignatureException {

	Key k = getPublicKey(signature, store);
	if (k == null) {
	    throw new NullPointerException("Key not specified or obtained");
	}

	return context.verify(signature, k).getCoreValidity();
    }

    private static Key getPublicKey(Element signature, KeyStore store)
	throws KeyStoreException, XSignatureException {

	Element e = KeyInfo.searchForKeyInfo(signature);
	Key key = null;
	if (e != null) {
	    KeyInfo ki = new KeyInfo(e);
	    if (key == null) {
		key = ki.getKeyValue();
	    }
	    if (key == null) {
		String[] nn = ki.getKeyNames();
		if (nn != null) {
		    for (int i = 0; i < nn.length; i++) {
			Certificate c = store.getCertificate(nn[i]);
			if (c != null) {
			    key = c.getPublicKey();
			    break;
			}
		    }
		}
	    }
	}

	return key;
    }
}
