/*
 * (C) Copyright IBM Corp. 1999-2000  All rights reserved.
 *
 * US Government Users Restricted Rights Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */

package xss4jsamples.asn1;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.Rectangle;
import java.awt.Toolkit;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowAdapter;

import java.io.File;

import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;

/**
  * ExternalDTDDialog is the dialog for using a external DTD.
  */
public class ExternalDTDDialog extends JDialog implements ActionListener {
    public static final int OK_BUTTON = 0; // return value if a OK button is chosen
    public static final int CANCEL_BUTTON = 1; // return value if a Cancel button is chosen
    public static final int CLOSE_BUTTON = 2; // return value if a window close button is chosen

    private JTextField asn1TextField,dtdFileTextField,dtdURLTextField,rootTextField;
    private JButton asn1ChooseButton,dtdChooseButton,okButton,cancelButton;
    private JRadioButton dtdFileRadioButton,dtdURLRadioButton;
    private JLabel dtdFileLabel,dtdURLLabel;
    private JFileChooser fileChooser;

    private String asn1File; // a name of a file containg a DER encoded ASN.1 data
    private String dtd; // a name of a file containing a DTD
    private String rootName; // a name of a root
    private int pressedButton; // a pressed button

    /**
      * Constructs a dialog for using a external DTD.
      *
      * @param <CODE>owner</CODE> - the frame from which the dialog is displayed.
      * @param <CODE>title</CODE> - the String to display in the dialog's title bar.
      * @param <CODE>modal</CODE> - true for a modal dialog, false for one that allows others windows to be active at the same time
      */
    public ExternalDTDDialog(Frame owner, String title, boolean modal) {
        super(owner, title, modal);

        getContentPane().setLayout(new BorderLayout(0, 20));
        setResizable(false);
        setSize(550, 260);
        Dimension screenDim = Toolkit.getDefaultToolkit().getScreenSize();
        Rectangle winDim = getBounds();
        setLocation((screenDim.width - winDim.width) / 2, (screenDim.height - winDim.height) / 2); 

        fileChooser = new JFileChooser();
        fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
        fileChooser.setFileHidingEnabled(false);
        fileChooser.setMultiSelectionEnabled(false);

        JPanel mainPanel = new JPanel();
        JPanel buttonPanel = new JPanel();
        getContentPane().add(new JPanel(), BorderLayout.NORTH);
        getContentPane().add(mainPanel, BorderLayout.CENTER);
        getContentPane().add(buttonPanel, BorderLayout.SOUTH);
        mainPanel.setLayout(new BorderLayout(0, 10));
        buttonPanel.setLayout(new FlowLayout());

        JPanel asn1Panel = new JPanel();
        JPanel dtdPanel = new JPanel();
        JPanel rootPanel = new JPanel();
        mainPanel.add(asn1Panel, BorderLayout.NORTH);
        mainPanel.add(dtdPanel, BorderLayout.CENTER);
        mainPanel.add(rootPanel, BorderLayout.SOUTH);
        asn1Panel.setLayout(new FlowLayout(FlowLayout.LEFT));
        dtdPanel.setLayout(new GridLayout(2, 1));
        rootPanel.setLayout(new FlowLayout(FlowLayout.LEFT));

        JPanel dtdFilePanel = new JPanel();
        JPanel dtdURLPanel = new JPanel();
        dtdPanel.add(dtdFilePanel);
        dtdPanel.add(dtdURLPanel);
        dtdFilePanel.setLayout(new FlowLayout(FlowLayout.LEFT));
        dtdURLPanel.setLayout(new FlowLayout(FlowLayout.LEFT));

        JLabel asn1Label = new JLabel("ASN.1 File :   ");
        asn1TextField = new JTextField(30);
        asn1ChooseButton = new JButton("Choose");
        asn1Panel.add(asn1Label);
        asn1Panel.add(asn1TextField);
        asn1Panel.add(asn1ChooseButton);
        asn1ChooseButton.addActionListener(this);

        ButtonGroup bg = new ButtonGroup();

        JLabel dtdLabel = new JLabel("DTD :   ");
        dtdFileRadioButton = new JRadioButton("");
        bg.add(dtdFileRadioButton);
        dtdFileLabel = new JLabel("File :   ");
        dtdFileTextField = new JTextField(30);
        dtdChooseButton = new JButton("Choose");
        dtdFilePanel.add(dtdLabel);
        dtdFilePanel.add(dtdFileRadioButton);
        dtdFilePanel.add(dtdFileLabel);
        dtdFilePanel.add(dtdFileTextField);
        dtdFilePanel.add(dtdChooseButton);
        dtdFileRadioButton.addActionListener(this);
        dtdChooseButton.addActionListener(this);

        dtdURLRadioButton = new JRadioButton("");
        bg.add(dtdURLRadioButton);
        dtdURLLabel = new JLabel("URL :   ");
        dtdURLTextField = new JTextField(30);
        dtdURLPanel.add(new JLabel("             "));
        dtdURLPanel.add(dtdURLRadioButton);
        dtdURLPanel.add(dtdURLLabel);
        dtdURLPanel.add(dtdURLTextField);
        dtdURLRadioButton.addActionListener(this);

        JLabel rootLabel = new JLabel("Root Name :   ");
        rootTextField = new JTextField(30);
        rootPanel.add(rootLabel);
        rootPanel.add(rootTextField);

        okButton = new JButton("OK");
        cancelButton = new JButton("Cancel");
        buttonPanel.add(okButton);
        buttonPanel.add(cancelButton);
        okButton.addActionListener(this);
        cancelButton.addActionListener(this);

        addWindowListener(new WindowAdapter() {
                public void windowClosing(WindowEvent we) {
                    pressedButton = CLOSE_BUTTON;
                    setVisible(false);
                }
            });
    }

    /**
      * Invoked when an action occurs.
      */
    public void actionPerformed(ActionEvent ae) {
        // if selected the DTD -> File radio button
        if (ae.getSource() == dtdFileRadioButton) {
            dtdFileLabel.setEnabled(true);
            dtdFileTextField.setEnabled(true);
            dtdChooseButton.setEnabled(true);
            dtdURLLabel.setEnabled(false);
            dtdURLTextField.setEnabled(false);

        // if selected the DTD -> URL radio button
        } else if (ae.getSource() == dtdURLRadioButton) {
            dtdFileLabel.setEnabled(false);
            dtdFileTextField.setEnabled(false);
            dtdChooseButton.setEnabled(false);
            dtdURLLabel.setEnabled(true);
            dtdURLTextField.setEnabled(true);

        // if pressed the ASN.1 File -> Choose button
        } else if (ae.getSource() == asn1ChooseButton) {
            int state = fileChooser.showOpenDialog(this);
            if (state == JFileChooser.APPROVE_OPTION) {
                File file = fileChooser.getSelectedFile();
                if (file != null) asn1TextField.setText(file.toString());
            }

        // if pressed the DTD -> File -> Choose button
        } else if (ae.getSource() == dtdChooseButton) {
            int state = fileChooser.showOpenDialog(this);
            if (state == JFileChooser.APPROVE_OPTION) {
                File file = fileChooser.getSelectedFile();
                if (file != null) dtdFileTextField.setText(file.toString());
            }

        // if pressed the OK button
        } else if (ae.getSource() == okButton) {
            asn1File = asn1TextField.getText();
            if (asn1File.equals("")) {
                JOptionPane.showMessageDialog(this, "ASN.1 File is Empty !", "", JOptionPane.ERROR_MESSAGE);
                return;
            }

            if (dtdFileRadioButton.isSelected()) dtd = dtdFileTextField.getText();
            else dtd = dtdURLTextField.getText();
            if (dtd.equals("")){
                JOptionPane.showMessageDialog(this, "DTD is Empty !","",JOptionPane.ERROR_MESSAGE);
                return;
            }

            rootName = rootTextField.getText();
            if (rootName.equals("")) {
                JOptionPane.showMessageDialog(this, "Root Name is Empty !", "", JOptionPane.ERROR_MESSAGE);
                return;
            }

            pressedButton = OK_BUTTON;
            setVisible(false);

        // if pressed the Cancel button
        } else if (ae.getSource() == cancelButton) {
            pressedButton = CANCEL_BUTTON;
            setVisible(false);
        }
    }

    /**
      * Returns the name of the file containing the DER encoded ASN.1 data.
      *
      * @return the name of the file containing the DER encoded ASN.1 data.
      */
    public String getASN1File() {
        return asn1File;
    }

    /**
      * Returns the name of the file containing the DTD.
      *
      * @return the name of the file containing the DTD.
      */
    public String getDTD() {
        return dtd;
    }

    /**
      * Returns the name of the root.
      *
      * @return the name of the root.
      */
    public String getRootName() {
        return rootName;
    }

    /**
      * Return the pressed button.
      *
      * @return the pressed button: OK_BUTTON, CANCEL_BUTTON, CLOSE_BUTTON.
      */
    public int getPressedButton() {
        return pressedButton;
    }

    /**
      * Initializes and shows the dialog.
      */
    public void showDialog() {
        init();
        setVisible(true);
    }

    /**
      * Initializes the dialog.
      */
    private void init() {
        asn1TextField.setText("");
        dtdFileTextField.setText("");
        dtdURLTextField.setText("");
        rootTextField.setText("");
        asn1File = "";
        dtd = "";
        rootName = "";

        dtdFileRadioButton.setSelected(true);
        dtdURLRadioButton.setSelected(false);
        dtdFileLabel.setEnabled(true);
        dtdFileTextField.setEnabled(true);
        dtdChooseButton.setEnabled(true);
        dtdURLLabel.setEnabled(false);
        dtdURLTextField.setEnabled(false);
    }
}
