/*
 * (C) Copyright IBM Corp. 1999-2000  All rights reserved.
 *
 * US Government Users Restricted Rights Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */

package xss4jsamples.asn1;

import java.awt.BorderLayout;
import java.awt.Font;

import java.io.IOException;
import java.io.Reader;
import java.io.Writer;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

import javax.swing.border.EtchedBorder;
import javax.swing.border.TitledBorder;

import javax.swing.text.BadLocationException;

/**
  * LogPanel is the panel for logs.
  */
public class LogPanel extends JPanel {
    private JTextArea logTextArea; // a text area for logs

    /**
      * Constructs a panel for logs.
      */
    public LogPanel() {
        super();
        setLayout(new BorderLayout());
        setBorder(new TitledBorder(new EtchedBorder()," Log Window "));

        logTextArea = new JTextArea();
        logTextArea.setFont(new Font("Monospaced", Font.PLAIN, 13));
        logTextArea.setEditable(false);
        add(new JScrollPane(logTextArea, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED), BorderLayout.CENTER);
    }

    /**
      * Appends the given text to the end of the document.
      *
      * @param <CODE>str</CODE> - the text to insert
      */
    public void append(String str) {
        logTextArea.append(str + "\n");
    }

    /**
      * Clears the contents in text area.
      *
      * @param <CODE>str</CODE> - the text to insert
      */
    public void clear() {
        logTextArea.setText("");
    }

    /**
      * Transfers the currently selected range in the associated text area to the system clipboard,
      * leaving the contents in the text area.
      */
    public void copy() {
        logTextArea.copy();
    }

    /**
      * Transfers the currently selected range in the associated text area to the system clipboard,
      * removing the contents from the text area. 
      */
    public void cut() {
        logTextArea.cut();
    }

    /**
      * Returns the text contained in the text area.
      *
      * @return the text.
      */
    public String getText() {
        return logTextArea.getText();
    }

    /**
      * Fetches a portion of the text represented by the text area.
      *
      * @param <CODE>offs</CODE> - the offset >= 0.
      * @param <CODE>offs</CODE> - the length >= 0.
      *
      * @return the text.
      */
    public String getText(int offs, int len) throws BadLocationException {
        return logTextArea.getText(offs, len);
    }

    /**
      * Inserts the specified text at the specified position.
      *
      * @param <CODE>str</CODE> - the text to insert.
      * @param <CODE>pos</CODE> - the position at which to insert >= 0.
      */
    public void insert(String str, int pos) {
        logTextArea.insert(str, pos);
    }
    
    /**
      * Transfers the contents of the system clipboard into the text area.
      */
    public void paste() {
        logTextArea.paste();
    }

    /**
      * Initializes the text area from a stream. By default this will load the text area 
      * as plain text. Previous contents of the text area are discarded.
      *
      * @param <CODE>in</CODE> - The stream to read from
      * @param <CODE>desc</CODE> - An object describing the stream. This might be a string, a File, a URL, etc. 
      *
      * @exception java.io.IOException -
      */
    public void read(Reader in, Object desc) throws IOException {
        logTextArea.read(in, desc);
    }

    /**
      * Selects all the text in the text area.
      */
    public void selectAll() {
        logTextArea.selectAll();
    }

    /**
      * Sets the contents of the text area to the specified text.
      *
      * @param <CODE>t</CODE> - the new text to be set.
      */
    public void setText(String t) {
        logTextArea.setText(t);
    }

    /**
      * Stores the contents of the text area into the given stream. 
      * By default this will store the text area as plain text.
      *
      * @param <CODE>out</CODE> - the output stream.
      *
      * @exception java.io.IOException -
      */
    public void write(Writer out) throws IOException {
        logTextArea.write(out);
    }
}
