/*
 * (C) Copyright IBM Corp. 1999-2000  All rights reserved.
 *
 * US Government Users Restricted Rights Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */

/*
 * Refer to LabelX509Cert.java and Translator.java
 * applied by mine@ax.jp.nec.com
 * added "root..NameConstraints" to OIDTable.
 */

package xss4jsamples.asn1;

import com.ibm.xml.asn1.ASN12SAX;
import com.ibm.xml.asn1.ASN1Syntax2DTD;
import com.ibm.xml.asn1.LabelSAX;
import com.ibm.xml.asn1.ParseException;
import com.ibm.xml.asn1.SAX2ASN1;
import com.ibm.xml.asn1.SAXEventDumper;
import com.ibm.xml.asn1.UnlabelSAX;
import com.ibm.xml.parser.DTD;
import com.ibm.xml.parser.Parser;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.io.Reader;
import java.io.Writer;
import java.util.Arrays;
import java.util.StringTokenizer;

import org.apache.xerces.parsers.SAXParser;
import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

/* 
 * getting from LabelX509Cert
 */ 
import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.helpers.AttributesImpl;
import org.xml.sax.helpers.DefaultHandler;

import com.ibm.util.Hex;

/**
 * This is a program to translate XML document create by LabelX509Cert
 * into DER encoded ASN.1 data.
 * 
 * @author Mine Sakurai
 * @author Takeshi IMAMURA
 */
public class UnlabelX509Cert implements ErrorHandler {
  private String opt;  // Option
  private String src;  // Source file
  private String des;  // Destination file

  private Unconverter unconv = null;
  private Shrinker2 srnk2 = null;
  private Shrinker1 srnk = null;
  private SAXParser psr = null;

  /**
   * Constructs an object and initializes some variables according to
   * the specified arguments.
   */
  public UnlabelX509Cert() {
    unconv = new Unconverter();

    srnk2 = new Shrinker2(unconv);
    srnk  = new Shrinker1(srnk2);

    psr = new SAXParser();
    psr.setContentHandler(srnk);
    psr.setErrorHandler(this);

    // Turn on validation because some versions of Xerces do not
    // attach any default attributes to elements by default (is this
    // bug?)
    try {
	psr.setFeature("http://apache.org/xml/features/validation/dynamic", true);

    } catch (SAXException e) {
	Exception e2 = e.getException();
	if (e2 == null) {
	    e2 = e;
	}
	e2.printStackTrace();
    }
  }

  /**
   * Unlabels a labeled XML X509Cert document and translates it into
   * DER encoded ASN.1 data.
   *
   * @exception java.io.IOException -
   * @exception org.xml.sax.SAXException -
   */
  public void start(String src, String des) {
    try {
      OutputStream out = System.out;
      if (des != null) {
        out = new FileOutputStream(des);
      }
      unconv.setOutputStream(out);

      psr.parse(src);

    } catch (Exception e){
      if (e instanceof SAXException){
        Exception e_ = ((SAXException)e).getException();
        if (e_ != null){
          e = e_;
        }
      }
      e.printStackTrace();
      System.exit(1);
    }
  }

  public class Unconverter extends DefaultHandler{
    private SAX2ASN1 s2a = null;
    private UnlabelSAX usax = null;

    public Unconverter() {
      s2a = new SAX2ASN1();
      usax = new UnlabelSAX(s2a);
    }

    public void setOutputStream(OutputStream out){
      s2a.setOutputStream(out);
    }
 
    public byte [] toByteArray(){
      return s2a.toByteArray();
    }

    public void startDocument() throws SAXException {
      usax.startDocument();
    }

    public void endDocument() throws SAXException {
      usax.endDocument();
    }

    public void startElement(String uri, String localName, String name, Attributes atts) throws SAXException {
      usax.startElement(uri, localName, name, atts);
    }

    public void endElement(String uri, String localName, String name) throws SAXException {
      usax.endElement(uri, localName, name);
    }

    public void characters(char chars[], int start, int length) throws SAXException {
      usax.characters(chars, start, length);
    }

    public void init () {
      usax.init();
    }
  }

  /**
   * Shrinks a part of labeled XML X509Cert document.
   * translates names into bits of KeyUsage Extension value.
   */
  public class Shrinker1 extends DefaultHandler {
    private ContentHandler hand = null;

    private StringBuffer buf  = null;
    private OIDTable     oidt = null;
    private String       tag  = null;

    private String uri      = null;
    private String lname    = null;
    private String name     = null;
    private Attributes atts = null;

    public Shrinker1(ContentHandler hand) {
      this.hand = hand;
    }

    public void startDocument() throws SAXException {
      hand.startDocument();
    }

    public void endDocument() throws SAXException {
      hand.endDocument();
    }

    public void startElement(String uri, String localName, String name, Attributes atts) throws SAXException {

      if (name.equals("root..KeyUsage")) {
	buf = new StringBuffer();

	this.uri = uri;
	lname = localName;
	this.name = name;
	this.atts = atts;

//      } else if (name.equals("root..NetscapeCertType")) {
//	buf = new StringBuffer();
//
//	this.uri = uri;
//	lname = localName;
//	this.name = name;
//	this.atts = atts;

      } else {
	hand.startElement(uri, localName, name, atts);
      }
    }

    public void endElement(String uri, String localName, String name) throws SAXException {

      if (name.equals("root..KeyUsage")) {
        String s = buf.toString();
	buf = null;
        //System.out.println(s);

	//
	// Convert names to bits
	//
        s = (new KeyUsage()).toBits(s);
        String bitsL = s.valueOf(s.length());
        s = com.ibm.xml.asn1.Base64.encodeBitString(s.toString());

        //
        // add the "encoding" and "length" attributes 
        //
        AttributesImpl aa = (AttributesImpl)atts;
        aa.addAttribute("", "", "encoding", "CDATA","base64");
        aa.addAttribute("", "", "length", "CDATA", bitsL);

	hand.startElement(this.uri, lname, this.name, aa);
	hand.characters(s.toCharArray(), 0, s.length());

//      } else if (name.equals("root..NetscapeCertType")) {
      }

      hand.endElement(uri, localName, name);
    }

    public void characters(char chars[], int start, int length) throws SAXException {
      if (buf == null) {
	hand.characters(chars, start, length);

      } else {
	buf.append(chars, start, length);
      }
    }
  }

  /**
   * Shrinks a part of labeled XML X509Cert document.
   * translates XML X509 Extension value into
   * DER encoded ASN.1 data.
   */
  public class Shrinker2 extends DefaultHandler {
    private ContentHandler hand = null;

    private StringBuffer buf  = null;
    private OIDTable     oidt = null;
    private String       tag  = null;
    private int          getchild = 0;

    private String uri      = null;
    private String lname    = null;
    private String name     = null;
    private Attributes atts = null;
    private Attributes patts = null;

    private Unconverter subunconv = null;

    public Shrinker2(ContentHandler hand) {
      this.hand = hand;


    }

    public void startDocument() throws SAXException {
      hand.startDocument();
    }

    public void endDocument() throws SAXException {
      hand.endDocument();
    }

    public void startElement(String uri, String localName, String name, Attributes atts) throws SAXException {
      if (name.equals("Extension.extnId")) {
        buf = new StringBuffer();

        hand.startElement(uri, localName, name, atts);

      } else if (name.equals("Extension.extnValue")) {
        buf = new StringBuffer();

        subunconv = new Unconverter();
        subunconv.init();

        this.uri = uri;
        lname = localName;
        this.name = name;
        this.atts = atts;

        if (tag != null){
          getchild = 1;

	  //
	  // add the "encoding" attribute if the attribute doesn't exist
	  //
          AttributesImpl aa = (AttributesImpl)atts;
          aa.addAttribute("", "", "encoding", "CDATA", "base64");
          
          hand.startElement(uri, localName, name, aa);

          subunconv.startDocument();
        } 

      } else if (getchild == 1) {
        subunconv.startElement("", "", name, atts);

      } else {
	hand.startElement(uri, localName, name, atts);

      }
    }

    public void endElement(String uri, String localName, String name) throws SAXException {
      if (name.equals("Extension.extnId")) {
        String s = buf.toString();  // Extension identifier
        buf = null;

	//
	// OID -> Tag name
	//
        if (oidt == null) {
          oidt = new OIDTable();
        }
        tag = oidt.lookup(s);
      
        hand.characters(s.toCharArray(), 0, s.length());
        hand.endElement(uri, localName, name);

      } else if (name.equals("Extension.extnValue")) {
        String s = buf.toString();  // Extension value
        buf = null;

        if (tag == null) {
          hand.startElement(this.uri, lname, this.name, atts);
          hand.characters(s.toCharArray(), 0, s.length());

        } else {
          subunconv.endDocument();

	  //
	  // Shrink the value
	  //
          String shrinkedvalue = Hex.toString(subunconv.toByteArray());
          String b64value = 
              com.ibm.xml.asn1.Base64.encodeOctetString(shrinkedvalue);
          hand.characters(b64value.toCharArray(), 0,b64value.length());

          getchild = 0; 
	}

        hand.endElement(uri, localName, name);

      } else if (getchild == 1) {
        subunconv.endElement("", "", name);

      }else{
        hand.endElement(uri, localName, name);
      }

  }

    public void characters(char chars[], int start, int length) throws SAXException {
      if (buf == null) {
	hand.characters(chars, start, length);

      } else if (getchild == 1) {
        subunconv.characters(chars, start, length);

      } else {
	buf.append(chars, start, length);
        //System.out.println("characters:"+buf.toString());
      }
    }
  }

  public class OIDTable {
    private final String TAGS[] = {null, null, null, null, null,
				   null, null, null, null,
				   "root..SubjectDirectoryAttributes",  // 9
				   null, null, null, null,
				   "root..SubjectKeyIdentifier",        // 14
				   "root..KeyUsage",                    // 15
				   "root..PrivateKeyUsagePeriod",       // 16
				   "root..SubjectAltName",              // 17
				   "root..IssuerAltName",               // 18
				   "root..BasicConstraints",            // 19
				   null, null, null, null, null,
				   null, null, null, null, null,
				   "root..NameConstraints",             // 30
				   "root..CRLDistPointsSyntax",         // 31
				   "root..CertificatePolicies",         // 21
				   "root..PolicyMappings",              // 33
				   null,
				   "root..AuthorityKeyIdentifier",      // 35
				   "root..PolicyConstraints",           // 36
				   "root..ExtKeyUsageSyntax",           // 37
				   null, null};

    // Netscape Private Extension OIDs 
    // http://home.netscape.com/eng/security/comm4-cert-exts.html
    //
    private final String netscapeTAGS[] = {null,
                                           "root..NetscapeCertType", //1
                                           null, null, null, null, 
                                           null, null, null, null,
                                           null, null, null, null,
                                           null};

    public String lookup(String oid) {
      String tag = null;

      if (oid.startsWith("2.5.29.")) {
	int i = Integer.parseInt(oid.substring("2.5.29.".length()));
	if (i > -1 && i < 40) {
	  tag = TAGS[i];
	}
//      } else if (oid.startsWith("2.16.840.1.113730.1.")) {
//        int i = Integer.parseInt(oid.substring("2.16.840.1.113730.1.".length()));
//        if (i > -1 && i < 15) {
//          tag = netscapeTAGS[i];
//        }
      }

      return tag;
    }
  }

  public class KeyUsage {
    private final String USAGE[] = {"digitalSignature",
				    "nonRepudiation",
				    "keyEncipherment",
				    "dataEncipherment",
				    "keyAgreement",
				    "keyCertSign",
				    "cRLSign",
				    "encipherOnly",
				    "decipherOnly" };

      public String toBits(String names) {
	  boolean[] bb = new boolean[USAGE.length];
	  Arrays.fill(bb, false);

	  StringTokenizer st = new StringTokenizer(names, ",");
	  while (st.hasMoreTokens()) {
	      int i = toIndex(st.nextToken());
	      if (i < 0) {
		  continue;
	      }
	      bb[i] = true;
	  }

	  StringBuffer sb = new StringBuffer();
	  for (int i = 0, j = 0; i < bb.length; i++) {
	      if (!bb[i]) {
		  continue;
	      }
	      for (; j <= i; j++) {
		  char c = '0';
		  if (j == i) {
		      c = '1';
		  }
		  sb.append(c);
	      }
	  }

	  return sb.toString();
      }

      private int toIndex(String name) {
	  int index = -1;
	  for (int i = 0; i < USAGE.length; i++) {
	      if (USAGE[i].equals(name)) {
		  index = i;
	      }
	  }

	  return index;
      }

//    public String toBits(String names) {
//      int numofKeyUsage = 9;
//      StringBuffer[] each_name = new StringBuffer[ numofKeyUsage];
//      //StringBuffer temp_name = new StringBuffer();
//      int j = 0; 
//      int numofbits = 1;
//
//      for (int i = 0; i < numofKeyUsage; i++){
//        each_name[ i] = new StringBuffer();
//      }
//
//      for (int i =0; i < names.length(); i++){
//
//        char c = names.charAt(i);
//
//        if (c == ','){
//          if (each_name[ j].equals(USAGE[ numofKeyUsage -1])){
//            numofbits = 2;
//          }
//          //System.out.println("j="+j+" "+each_name[ j].toString());
//          j++;
//        }else{
//          each_name[ j].append(c);
//        } 
//      }
//      //System.out.println("j="+j+" "+each_name[ j].toString());
//      j++;
//
//      StringBuffer bits = new StringBuffer();
//      bits.setLength(numofbits*8); 
//      for (int k = 0; k < bits.length(); k++){
//        bits.setCharAt(k,'0');          
//      }
//
//      for (int i = 0; i < j; i++){
//         String each = "";
//         each = each_name[i].toString();
//         //System.out.println(each);
//
//         for (int k = 0; k < numofKeyUsage; k++){
//           if (each.equals(USAGE[k])){
//             bits.setCharAt(k,'1');  
//           }
//         }
//      } 
//
//      return bits.toString();
//    }
  }

  public class NetscapeCertType {
    private final String USAGE[] = {"SSL client",
				    "SSL server",
				    "S/MIME",
				    "Object Signing",
				    "Reserved",
				    "SSL CA",
				    "S/MIME CA",
				    "Object Signing CA" };

    public String toNames(String bits) {
      StringBuffer names = new StringBuffer();

      String s = "";
      for (int i = 0; i < 8; i++) {
	if (bits.charAt(i) == '1') {
	  names.append(s+USAGE[i]);
	  s = ",";
	}
      }

      return names.toString();
    }
  }


  //
  // ErrorHandler methods
  //

  /**
   * Receive notification of a warning.  The warning information is
   * printed.
   *
   * @param <CODE>exc</CODE> the warning information encapsulated in a
   * SAX parse exception.
   */
  public void warning(SAXParseException exc) {
    System.err.println("[Warning] "+getLocation(exc)+": "+exc.getMessage());
  }

  /**
   * Receive notification of a recoverable error.  The error
   * information is printed.
   *
   * @param <CODE>exc</CODE> the error information encapsulated in a
   * SAX parse exception.
   */
  public void error(SAXParseException exc) {
    System.err.println("[Error] "+getLocation(exc)+": "+exc.getMessage());
  }

  /**
   * Receive notification of a non-recoverable error.  The error
   * information is printed and the exception is thrown again.
   *
   * @param <CODE>exc</CODE> the error information encapsulated in a
   * SAX parse exception.
   *
   * @exception org.xml.sax.SAXException -
   */
  public void fatalError(SAXParseException exc) throws SAXException {
    System.err.println("[Fatal Error] "+getLocation(exc)+": "+exc.getMessage());
    throw exc;
  }

  /**
   * Gets the location where the exception is thrown.
   *
   * @param <CODE>exc</CODE> the warning or error information
   * encapsulated in a SAX parse exception.
   */
  private String getLocation(SAXParseException exc) {
    StringBuffer buf = new StringBuffer();

    String id = exc.getSystemId();
    if (id != null) {
      int i = id.lastIndexOf('/');
      if (i != -1) {
        id = id.substring(i+1);
      }
      if (!id.equals("")) {
	buf.append(id+", ");
      }
    }
    buf.append(exc.getLineNumber()+", ");
    buf.append(exc.getColumnNumber());

    return buf.toString();
  }


  //
  // Main method
  //

  /**
   * Crates this object and starts a translation.
   */
  public static void main(String args[]) {
    String src = null;
    String des = null;

    int i = 0;
    if (i < args.length) {
      src = args[i++];
    }

    if (i < args.length){
      des = args[i];
    }
    if (src == null){
      System.err.println("Usage: java asn1.UnlabelX509Cert <xmlfile> [<des>]");
      System.exit(1);
    }
    UnlabelX509Cert unlab = new UnlabelX509Cert();
    unlab.start(src, des);
  }
}
