/*
 * (C) Copyright IBM Corp. 1999,2000,2001  All rights reserved.
 *
 * US Government Users Restricted Rights Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */

package xss4jsamples.c14n;

import com.ibm.dom.util.ExclusiveCanonicalizer;
import com.ibm.dom.util.StandardErrorHandler;
import com.ibm.dom.util.XPathCanonicalizer;
import java.io.OutputStreamWriter;
import java.io.Writer;
import org.apache.xerces.parsers.DOMParser;
import org.w3c.dom.Document;
import org.xml.sax.InputSource;

/**
 *
 * @version $Id: C14nDOM.java,v 1.5 2004/04/14 06:41:14 kent Exp $
 * @author TAMURA Kent &lt;kent@trl.ibm.co.jp&gt;
 */
public class C14nDOM {

    public static void main(String[] argv) throws Exception {
        boolean withComments = false;
        boolean exclusive = false;

        for (int i = 0;  i < argv.length;  i++) {
            if (argv[i].equals("-xpath")) {
                withComments = false;
                exclusive = false;
            } else if (argv[i].equals("-xpathWC")) {
                withComments = true;
                exclusive = false;
            } else if (argv[i].equals("-exclusive")) {
                exclusive = true;
                withComments = false;
            } else if (argv[i].equals("-exclusiveWC")) {
                exclusive = true;
                withComments = true;
            } else {
                System.err.println("Unknown option: "+argv[i]);
            }
        }
        if (exclusive)
            System.err.println("Use Exclusive XML Canonicalization http://www.w3.org/Signature/Drafts/xml-exc-c14n");
        else
            System.err.println("Use REC-xml-c14n-20010315.");

        //
        // Parses an XML document.
        //
        DOMParser domp = new DOMParser();
        domp.setFeature("http://xml.org/sax/features/validation", true);
        domp.setErrorHandler(new StandardErrorHandler());
        domp.parse(new InputSource(System.in));
        Document doc = domp.getDocument();

        //
        // Outputs as the canonical XML.
        //
        Writer wr = new OutputStreamWriter(System.out, "UTF-8");
        if (exclusive) {
            //ExclusiveCanonicalizer.serializeSubset(null, XPathCanonicalizer.toNodeset(doc, null, withComments), withComments, wr);
            ExclusiveCanonicalizer.serializeAll(null, doc, withComments, wr);
        } else {
            XPathCanonicalizer.serializeAll(doc, withComments, wr);
        }
        wr.close();
    }
}

