/*
 * (C) Copyright IBM Corp. 2000,2001,2002  All rights reserved.
 *
 * US Government Users Restricted Rights Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */

package xss4jsamples.c14n;

import com.ibm.dom.util.DOMUtil;
import com.ibm.dom.util.ExclusiveCanonicalizer;
import com.ibm.dom.util.StandardErrorHandler;
import com.ibm.dom.util.XPathCanonicalizer;
import com.ibm.xml.dsig.util.DOMParserNS;
import com.ibm.xml.dsig.util.XPathUtil;
import java.io.OutputStreamWriter;
import java.io.StringReader;
import java.io.Writer;
import java.util.Hashtable;
import javax.xml.parsers.DocumentBuilder;
import org.apache.xml.utils.PrefixResolverDefault;
import org.apache.xpath.XPath;
import org.apache.xpath.XPathContext;
import org.apache.xpath.objects.XObject;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.traversal.NodeIterator;
import org.xml.sax.InputSource;

/**
 *
 * @version $Id: SerializeSubset.java,v 1.6 2004/04/14 06:41:15 kent Exp $
 * @author TAMURA Kent &lt;kent@trl.ibm.co.jp&gt;
 */
public class SerializeSubset {
    public static void main(String[] argv) throws Exception {
        if (argv.length != 1 && argv.length != 2) {
            System.err.println("Usage: SerializeSubset <XPath expression> < <XML instance>");
            System.err.println("Usage: SerializeSubset -xml '<foo xmlns:hoge=\"...\">...XPath expression...</foo>' < <XML instance>");
            System.exit(1);
        }
        String xpathExpression = null;
        String xpathxml = null;
        boolean exclusive = false;
        boolean dump = false;
        Hashtable prefixList = null; // unpl for exclusive c14n
        for (int i = 0;  i < argv.length;  i++) {
            if (argv[i].equals("-xml")) {
                xpathxml = argv[++i];
            } else if (argv[i].equals("-dump")) {
                dump = true;
            } else if (argv[i].startsWith("-e")) {
                exclusive = true;
                prefixList = ExclusiveCanonicalizer.parsePrefixList(argv[i].substring(2));
            } else {
                xpathExpression = argv[i];
            }
        }

        //
        // Parses an XML document.
        //
        DocumentBuilder domp = DOMParserNS.createBuilder();
        domp.setErrorHandler(new StandardErrorHandler());
        Document doc = domp.parse(new InputSource(System.in));
        XPathUtil.fixTree(doc);
        //dsig.SignatureUtil.printDocument(doc, System.out);

        XObject xobj;
        //
        // Evaluates specified XPath expression against the document.
        //
        if (xpathExpression != null) {
            xobj = XPathUtil.evalXPath(doc, xpathExpression);
        } else {
            Document xpdoc = domp.parse(new InputSource(new StringReader(xpathxml)));
            xpathExpression = DOMUtil.getStringValue(xpdoc.getDocumentElement());
            PrefixResolverDefault presolver = new PrefixResolverDefault(xpdoc.getDocumentElement());
            XPath xpath = new XPath(xpathExpression, null, presolver, XPath.SELECT);
            xobj = xpath.execute(new XPathContext(), doc, presolver);
        }

        //
        // Serializes selected subset
        //
        Writer wr = new OutputStreamWriter(System.out, "UTF-8");
        if (xobj.getType() != XObject.CLASS_NODESET) {
            wr.write(xobj.str());
        } else {
            if (dump) {
                NodeIterator iter = xobj.nodeset();
                Node n;
                while ((n = iter.nextNode()) != null) {
                    System.err.println(n.getNodeName());
                }
                iter.detach();
            }
            NodeIterator iter = xobj.nodeset();
            if (exclusive) {
                ExclusiveCanonicalizer.serializeSubset(prefixList, iter, false, wr);
            } else {
                XPathCanonicalizer.serializeSubset(iter, false, wr);
            }
            iter.detach();
        }
        wr.close();
    }
}

