/*
 * (C) Copyright IBM Corp. 2000,2001  All rights reserved.
 *
 * US Government Users Restricted Rights Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */

package xss4jsamples.dsig;

import com.ibm.xml.dsig.AlgorithmFactory;
import com.ibm.xml.dsig.Transform;
import com.ibm.xml.dsig.TransformContext;
import com.ibm.xml.dsig.TransformException;
import com.ibm.dom.util.Digest;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import org.w3c.dom.Node;

/**
 * This is a sample class that shows extensibility of this XML Signature library.
 * The class enable for application
 *
 * @version $Id: AlgorithmExtension.java,v 1.1 2001/04/19 02:56:57 kent Exp $
 * @author TAMURA Kent &lt;kent@trl.ibm.co.jp&gt;
 */
public class AlgorithmExtension extends AlgorithmFactory {

    static class DOMHASHTransform extends Transform {
        public static final String URI = "http://www.ibm.com/xml/dsig/transform/domhash-sha1";

        DOMHASHTransform() {
            super();
        }

        public String getURI() {
            return URI;
        }

        public void transform(TransformContext context) throws TransformException {
            try {
                Node node = context.getNode();
                MessageDigest md = MessageDigest.getInstance("SHA");
                context.setContent(Digest.getDigest(md, node), null);
            } catch (NoSuchAlgorithmException nsae) {
                throw TransformException.create(nsae);
            }
        }
    }
    
    public AlgorithmExtension(String provider) {
        super(provider);
    }

    public Transform getTransform(String uri) throws NoSuchAlgorithmException {
        if (uri != null && uri.equals(DOMHASHTransform.URI))
            return new DOMHASHTransform();
        return super.getTransform(uri);
    }
}
