/*
 * (C) Copyright IBM Corp. 2002  All rights reserved.
 *
 * US Government Users Restricted Rights Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */

package xss4jsamples.dsig;

import com.ibm.dom.util.XPathCanonicalizer;
import com.ibm.xml.dsig.KeyInfo;
import java.io.File;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.security.cert.X509Certificate;
import org.w3c.dom.Document;

/**
 * Utility functions for samples.
 *
 * @author TAMURA Kent
 * @version $Id: SignatureUtil.java,v 1.1 2002/03/11 09:51:22 kent Exp $
 */
public class SignatureUtil {
    private SignatureUtil() {
    }

    public static String getDefaultKeystoreName() {
        return System.getProperty("user.home")+File.separator+".keystore";
    }

    public static KeyInfo createKeyInfo(X509Certificate cert) {
        KeyInfo keyInfo = new KeyInfo();
        KeyInfo.X509Data x5data = new KeyInfo.X509Data();
        x5data.setCertificate(cert);
        x5data.setParameters(cert, true, true, true);
        keyInfo.setX509Data(new KeyInfo.X509Data[] { x5data });
        keyInfo.setKeyValue(cert.getPublicKey());
        return keyInfo;
    }

    public static void printDocument(Document doc, OutputStream out)
            throws IOException {
        Writer wr = new OutputStreamWriter(out, "UTF-8");
        XPathCanonicalizer.serializeAll(doc, true, wr);
        wr.flush();
        // Serizlizer of Xerces 2.0.0 breaks signatures.
        //OutputFormat format = new OutputFormat("xml", "UTF-8", false);
        //format.setPreserveSpace(true);
        //new XMLSerializer(wr, format).serialize(doc);
    }

}
