/*
 * (C) Copyright IBM Corp. 2000,2001,2002  All rights reserved.
 *
 * US Government Users Restricted Rights Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */

package xss4jsamples.dsig;

import com.ibm.dom.util.StandardErrorHandler;
import com.ibm.xml.dsig.ResourceShower;
import com.ibm.xml.dsig.SignatureContext;
import com.ibm.xml.dsig.XSignatureException;
import com.ibm.xml.dsig.util.DOMParserNS;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Writer;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.ParserConfigurationException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.xml.sax.SAXException;

/**
 * Caluclates digest values of References in a Manifest.
 *
 * @version $Id: TemplateManifest.java,v 1.6 2004/04/14 06:41:16 kent Exp $
 * @author TAMURA Kent &lt;kent@trl.ibm.co.jp&gt;
 */
public class TemplateManifest {
    public static void main(String[] argv)
            throws IOException, SAXException, XSignatureException,
            ParserConfigurationException {

        ResourceShower shower = null;
        String templateURI = null;
        String filename = null;

        for (int i = 0;  i < argv.length;  i++) {
            if (argv[i].equals("-p")) {
                shower = new TemplateSign.StderrResourceShower();
            } else if (argv[i].equals("-o")) {
                if (++i >= argv.length) {
                    System.err.println("-o requires a file name");
                } else {
                    filename = argv[i];
                }
            } else {
                templateURI = argv[i];
            }
        }
        if (templateURI == null) {
            System.err.println("Usage: TemplateManifest [-p] <template-uri>");
            System.exit(1);
        }

        //
        // Initialize: Reading a manfest template.
        //
        DocumentBuilder domp = DOMParserNS.createBuilder();
        domp.setErrorHandler(new StandardErrorHandler());
        Document doc = domp.parse(templateURI);

        //
        // Calculate
        //
        long startSign = System.currentTimeMillis();
        SignatureContext sigContext = new SignatureContext();
        sigContext.setResourceShower(shower);
        Element manifestElement = doc.getDocumentElement();
        sigContext.digestManifest(manifestElement);
        long endSign = System.currentTimeMillis();
        System.err.println("Digest: "+(endSign-startSign)+"ms");

        //
        // Output
        //
        if (filename == null) {
            xss4jsamples.dsig.SignatureUtil.printDocument(doc, System.out);
        } else {
            FileOutputStream os = new FileOutputStream(filename);
            xss4jsamples.dsig.SignatureUtil.printDocument(doc, os);
            os.close();
        }
    }
}
