/*
 * (C) Copyright IBM Corp. 1999,2000,2001,2002  All rights reserved.
 *
 * US Government Users Restricted Rights Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */

package xss4jsamples.dsig;

import com.ibm.dom.util.StandardErrorHandler;
import com.ibm.dom.util.XPathCanonicalizer;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.security.InvalidKeyException;
import java.security.Key;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.UnrecoverableKeyException;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.util.Hashtable;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import com.ibm.xml.dsig.AlgorithmFactory;
import com.ibm.xml.dsig.KeyInfo;
import com.ibm.xml.dsig.ResourceShower;
import com.ibm.xml.dsig.SignatureContext;
import com.ibm.xml.dsig.SignatureStructureException;
import com.ibm.xml.dsig.XSignature;
import com.ibm.xml.dsig.XSignatureException;
import com.ibm.xml.dsig.util.AdHocIDResolver;
import com.ibm.xml.dsig.util.DOMParserNS;

/**
 * Sample signer.
 *
 * @version $Id: TemplateSign.java,v 1.10 2004/04/14 06:41:16 kent Exp $
 * @author TAMURA Kent &lt;kent@trl.ibm.co.jp&gt;
 */
public class TemplateSign {
    public static void main(String[] argv) throws Exception {
        String keystorepath = xss4jsamples.dsig.SignatureUtil.getDefaultKeystoreName();
        String keystoretype = "JKS";
        String alias = null;
        char[] storepass = null;
        char[] keypass = null;
        String templateURI = null;
        String outputfile = null;
        boolean verbose = false;

        int i;
        for (i = 0;  i < argv.length;  i++) {
            if (argv[i].equals("-keystore") && i+1 < argv.length) {
                keystorepath = argv[++i];
            } else if (argv[i].equals("-keystoretype") && i+1 < argv.length) {
                keystoretype = argv[++i];
            } else if (argv[i].equals("-o") && i+1 < argv.length) {
                outputfile = argv[++i];
            } else if (argv[i].equals("-v")) {
                verbose = true;
            } else if (alias == null) {
                alias = argv[i];
            } else if (storepass == null) {
                storepass = argv[i].toCharArray();
            } else if (keypass == null) {
                keypass = argv[i].toCharArray();
            } else if (templateURI == null) {
                templateURI = argv[i];
            }
        }
        if (templateURI == null || i < argv.length) {
            System.err.println("");
            System.err.println("Usage: TemplateSign [-keystore <path-to-keystore-file>] [-keystoretype <type>] <key-alias> <store-passwd> <key-passwd> <template-uri>");
            System.err.println("Options:");
            System.err.println("\t-keystoretype <type>\tThe type of keystore like 'JKS' 'JCEKS'. The default is 'JKS'");
            System.exit(1);
        }
        System.err.println("Key store: "+keystorepath);

        initTimes();
        sign(keystorepath, keystoretype, alias, storepass, keypass,
             templateURI, outputfile, verbose);
        printTimes();

        if (false) {
            initTimes();
            for (int j = 0;  j < 100;  j++) {
                sign(keystorepath, keystoretype, alias, storepass, keypass,
                     templateURI, outputfile, verbose);
            }
            printTimes();
        }
    }

    static long parseTime;
    static long xsignTime;
    static long outputTime;
    static long totalTime;
    static void initTimes() {
        parseTime = 0;  xsignTime = 0;  outputTime = 0;  totalTime = 0;
        XSignature.signTime = 0;
        XSignature.canonicalizeTime = 0;
        XSignature.resetDigestTime();
    }
    static void printTimes() {
        System.err.println("Total:             "+totalTime+"ms");
        System.err.println("  XML Parsing:       "+parseTime+"ms");
        System.err.println("  XSignature.sign(): "+xsignTime+"ms");
        System.err.println("    Digest for Ref:   "+XSignature.getDigestTime()+"ms");
        System.err.println("    Canonicalize:     "+XSignature.canonicalizeTime+"ms");
        System.err.println("    Sign with JCA/JCE:"+XSignature.signTime+"ms");
        System.err.println("  Serializing DOM:   "+outputTime+"ms");
    }

    static void sign(String keystorepath, String keystoretype, String alias,
                     char[] storepass, char[] keypass, String templateURI,
                     String outputfile, boolean verbose)
            throws IOException, KeyStoreException, CertificateException,
            NoSuchAlgorithmException, UnrecoverableKeyException,
            SignatureStructureException, SAXException, XSignatureException,
            ParserConfigurationException
        {

        long start = System.currentTimeMillis();
        //
        // Initialize: Reading a template signature.
        //
        DocumentBuilder builder = DOMParserNS.createBuilder();
        builder.setErrorHandler(new StandardErrorHandler());
        Document doc = builder.parse(templateURI);
        long parseEnd = System.currentTimeMillis();
        parseTime += parseEnd-start;

        //
        // Setup a certificate and a private key.
        //
        KeyStore keystore = KeyStore.getInstance(keystoretype);
        keystore.load(new FileInputStream(keystorepath), storepass);
        X509Certificate cert = (X509Certificate)keystore.getCertificate(alias);
        Key key = keystore.getKey(alias, keypass); // a private key
        if (key == null) {
            throw new RuntimeException("Could not get a key: "+alias);
        }

        //
        // Sign
        //
        NodeList list = doc.getElementsByTagNameNS(XSignature.XMLDSIG_NAMESPACE, "Signature");
        if (list.getLength() == 0) {
            System.err.println("No Signature element.");
        }
        long endSign = System.currentTimeMillis();
        SignatureContext sigContext = new SignatureContext();
        sigContext.setIDResolver(new AdHocIDResolver(doc));
        if (verbose)
            sigContext.setResourceShower(new StderrResourceShower());
        //sigContext.setAlgorithmFactory(new AlgorithmFactory("IBMJCE"));
        KeyInfo keyInfo = xss4jsamples.dsig.SignatureUtil.createKeyInfo(cert);
        for (int i = 0;  i < list.getLength();  i ++) {
            long startSign = System.currentTimeMillis();
            Element signatureElement = (Element)list.item(i);
            keyInfo.insertTo(signatureElement);
            sigContext.sign(signatureElement, key);
            endSign = System.currentTimeMillis();
            xsignTime += endSign-startSign;
        }

        //
        // Output
        //
        System.err.println("Output to: "+outputfile);
        if (outputfile == null) {
            xss4jsamples.dsig.SignatureUtil.printDocument(doc, System.out);
        } else {
            OutputStream os = new FileOutputStream(outputfile);
            xss4jsamples.dsig.SignatureUtil.printDocument(doc, os);
            os.close();
        }
        long endTime = System.currentTimeMillis();
        outputTime += endTime-endSign;
        totalTime += endTime-start;
    }

    static class StderrResourceShower implements ResourceShower {
        public StderrResourceShower() {
        }
        public void showSignedResource(Element owner, int count, String uri, String type,
                                       byte[] content, String encoding) {
            System.err.print("******** "+uri+" ********");
            try {
                if (encoding != null) {
                    System.err.print(new String(content, encoding));
                } else {
                    System.err.write(content);
                }
            } catch (IOException ioe) {
                ioe.printStackTrace();
            }
            System.err.println("****************");
        }
    }
}
