/*
 * (C) Copyright IBM Corp. 1999,2000,2001,2002  All rights reserved.
 *
 * US Government Users Restricted Rights Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */

package xss4jsamples.dsig;

import com.ibm.dom.util.StandardErrorHandler;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.security.Key;
import java.security.SecureRandom;
import java.util.Hashtable;
import javax.xml.parsers.DocumentBuilder;
import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
import org.w3c.dom.Document;                    // This requires DOM L2!
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import com.ibm.xml.dsig.ResourceShower;
import com.ibm.xml.dsig.SignatureContext;
import com.ibm.xml.dsig.XSignature;
import com.ibm.xml.dsig.util.AdHocIDResolver;
import com.ibm.xml.dsig.util.DOMParserNS;
import com.ibm.xml.dsig.util.SimpleKey;

/**
 * Sample signer for HMAC.
 *
 * @version $Id: TemplateSignHMAC.java,v 1.6 2004/04/14 06:41:17 kent Exp $
 * @author TAMURA Kent &lt;kent@trl.ibm.co.jp&gt;
 */
public class TemplateSignHMAC {
    public static void main(String[] argv) throws Exception {
        String templateURI = null;
        String keyString = null;
        String keyOutput = null;
        String keyFileName = null;
        String outputfile = null;
        for (int i = 0;  i < argv.length;  i++) {
            if (argv[i].equals("-genkey")) {
                keyOutput = argv[++i];
            } else if (argv[i].equals("-keystring")) {
                keyString = argv[++i];
            } else if (argv[i].equals("-keyfile")) {
                keyFileName = argv[++i];
            } else if (argv[i].equals("-o")) {
                outputfile = argv[++i];
            } else {
                templateURI = argv[i];
            }
        }
        if (templateURI == null ||
            (keyString == null && keyOutput == null && keyFileName == null)) {
            System.err.println("Usage: TemplateSignHMAC <template-uri> -genkey <generated-symmetric-key-filename> -o <outputfile>");
            System.err.println("       TemplateSignHMAC <template-uri> -keystring <ASCII string> -o <outputfile>");
            System.exit(1);
        }
        

        //
        // Initialize: Reading a template signature.
        //
        DocumentBuilder domp = DOMParserNS.createBuilder();
        domp.setErrorHandler(new StandardErrorHandler());
        Document doc = domp.parse(templateURI);

        //
        // Generate a key
        //
        SimpleKey key;
        byte[] rawKey;

        if (keyFileName != null) {
            File keyfile = new File(keyFileName);
            int length = (int)keyfile.length();
            System.err.print("Reading a key file ...");
            rawKey = new byte[length];
            FileInputStream fis = new FileInputStream(keyfile);
            fis.read(rawKey);
            fis.close();
            System.err.println(" done.");
        } else if (keyString != null) {
            rawKey = keyString.getBytes();
        } else {
            System.err.print("Generating a key ... ");
            SecureRandom randomGen = new SecureRandom();
            rawKey = new byte[512/8];
            randomGen.nextBytes(rawKey);
            System.err.println("done.");

            System.err.print("Writing the key to "+keyOutput+" ... ");
            OutputStream os = new FileOutputStream(keyOutput);
            os.write(rawKey);
            os.close();
            System.err.println("done.");
        }
        key = new SimpleKey(rawKey);
        System.err.println("The length of the key is "+(rawKey.length*8)+" bit.");
        
        //
        // Sign
        //
        NodeList list = doc.getElementsByTagNameNS(XSignature.XMLDSIG_NAMESPACE, "Signature");
        if (list.getLength() == 0) {
            System.err.println("No Signature element.");
        }
        SignatureContext sigContext = new SignatureContext();
        sigContext.setIDResolver(new AdHocIDResolver(doc));
        sigContext.setResourceShower(new TemplateSign.StderrResourceShower());
        for (int i = 0;  i < list.getLength();  i ++) {
            long startSign = System.currentTimeMillis();
            Element signatureElement = (Element)list.item(i);
            sigContext.sign(signatureElement, key);
            long endSign = System.currentTimeMillis();
            System.err.println("Sign: "+(endSign-startSign)+"ms");
        }

        //
        // Output
        //
        OutputStream stream;
        if (outputfile == null) {
            stream = System.out;
        } else {
            stream = new FileOutputStream(outputfile);
        }
        SignatureUtil.printDocument(doc, stream);
    }
}
