/*
 * (C) Copyright IBM Corp. 2000,2001  All rights reserved.
 *
 * US Government Users Restricted Rights Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */

package xss4jsamples.dsig;

import com.ibm.dom.util.StandardErrorHandler;
import com.ibm.xml.dsig.KeyInfo;
import com.ibm.xml.dsig.ResourceShower;
import com.ibm.xml.dsig.SignatureContext;
import com.ibm.xml.dsig.Validity;
import com.ibm.xml.dsig.XSignature;
import com.ibm.xml.dsig.util.AdHocIDResolver;
import com.ibm.xml.dsig.util.SimpleKey;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.security.Key;
import java.security.cert.CertificateException;
import java.security.cert.X509CRL;
import java.security.cert.X509Certificate;
import java.util.Set;
import org.apache.xerces.parsers.DOMParser;
import org.w3c.dom.Document;                    // DOM Level 2!
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

/**
 *
 * @version $Id: VerifyCUI.java,v 1.4 2004/04/14 06:44:34 kent Exp $
 * @author TAMURA Kent &lt;kent@trl.ibm.co.jp&gt;
 */
public class VerifyCUI {

    public static void main(String[] argv) throws Exception {
        InputStream is = null;
        int saveno = -2;
        String savename = null;
        Key key = null;
        boolean verbose = false;
        //
        // Analyze the command line
        //
        for (int i = 0;  i < argv.length;  i ++) {
            if (argv[i].equals("-s")) {
                saveno = Integer.parseInt(argv[i+1]);
                savename = argv[i+2];
                i += 2;
            } else if (argv[i].equals("-v")) {
                verbose = true;
            } else if (argv[i].equals("-keyfile")) {
                File keyfile = new File(argv[i+1]);
                int length = (int)keyfile.length();
                System.err.print("Reading a key file ...");
                byte[] rawkey = new byte[length];
                try {
                    FileInputStream fis = new FileInputStream(keyfile);
                    fis.read(rawkey);
                    fis.close();
                    System.err.println(" done: "+(length*8)+" bit");
                    key = new SimpleKey(rawkey);
                } catch (IOException ioe) {
                    System.err.println(ioe);
                }
                i++;
            } else {
                try {
                    is = new FileInputStream(argv[i]);
                    break;
                } catch (IOException ioe) {
                    System.err.println(ioe);
                }
            }
        }
        if (is == null)  is = System.in;

        //
        // Parses a signature document.
        //
        InputSource isrc = new InputSource(is);
        //isrc.setEncoding("UTF-8");
        Document doc = VerifyGUI.parse(isrc);

        //
        // Searches for a Signature element
        //
        NodeList nodeList = doc.getElementsByTagNameNS(XSignature.XMLDSIG_NAMESPACE,
                                                           "Signature");
        if (nodeList.getLength() == 0) {
            throw new RuntimeException("ERROR: The specified document has no Siganture element.");
        }
        Element signature = (Element)nodeList.item(0);

        SignatureContext sigContext = new SignatureContext();
        sigContext.setIDResolver(new AdHocIDResolver(doc));
        sigContext.setResourceShower(saveno <= -2 ? null : new VerifyGUI.ShowerImpl(saveno, savename));

        Element keyInfoElement = KeyInfo.searchForKeyInfo(signature);
        Validity validity = key != null
            ? VerifyGUI.verify(sigContext, signature, key, verbose)
            : VerifyGUI.verify(sigContext, signature, keyInfoElement, verbose);

        System.out.println(validity.getCoreValidity() ? "Core Validity: OK"
                           : "Core Validity: NG");
        System.out.println(validity.getSignedInfoValidity() ? "Signature Validity: OK"
                           : "Signature Validity: NG ("+validity.getSignedInfoMessage()+")");
        for (int i = 0; i < validity.getNumberOfReferences(); i++) {
            System.out.print("["+i+"] ");
            System.out.print("\""+validity.getReferenceURI(i)+"\"\t");
            if (validity.getReferenceValidity(i)) {
                System.out.println("OK");
            } else {
                System.out.println("NG: "+validity.getReferenceMessage(i));
            }
        }
        if (!validity.getCoreValidity())
            throw new RuntimeException("Core Validity: NG");
    }
}

