/*
 * (C) Copyright IBM Corp. 2000,2001,2002  All rights reserved.
 *
 * US Government Users Restricted Rights Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */

package xss4jsamples.dsig;

import com.ibm.dom.util.StandardErrorHandler;
import com.ibm.xml.dsig.SignatureContext;
import com.ibm.xml.dsig.Validity;
import com.ibm.xml.dsig.XSignature;
import com.ibm.xml.dsig.util.AdHocIDResolver;
import com.ibm.xml.dsig.util.DOMParserNS;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import javax.xml.parsers.DocumentBuilder;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

/**
 *
 * @version $Id: VerifyManifestCUI.java,v 1.2 2004/04/14 06:41:17 kent Exp $
 * @author TAMURA Kent &lt;kent@trl.ibm.co.jp&gt;
 */
public class VerifyManifestCUI {

    public static void main(String[] argv) throws Exception {
        //
        // Obtain input
        //
        InputStream is = null;
        for (int i = 0;  i < argv.length;  i ++) {
            is = new FileInputStream(argv[i]);
            break;
        }
        if (is == null)  is = System.in;

        //
        // Parses a manifest document.
        //
        DocumentBuilder domp = DOMParserNS.createBuilder();
        domp.setErrorHandler(new StandardErrorHandler());
        Document doc = domp.parse(new InputSource(is));

        //
        // Searches for a Manifest element
        //
        NodeList nodeList = doc.getElementsByTagNameNS(XSignature.XMLDSIG_NAMESPACE,
                                                       "Manifest");
        if (nodeList.getLength() == 0) {
            throw new RuntimeException("The specified document has no Manifest element.");
        }
        Element manifest = (Element)nodeList.item(0);

        //
        // Checks validity of the Manifest element
        //
        SignatureContext sigContext = new SignatureContext();
        sigContext.setIDResolver(new AdHocIDResolver(doc));
        Validity validity = sigContext.verifyManifest(manifest);

        //
        // Prints result
        //
        int invalid_counter = 0;
        for (int i = 0; i < validity.getNumberOfReferences(); i++) {
            System.out.print("\""+validity.getReferenceURI(i)+"\"\t");
            if (validity.getReferenceValidity(i)) {
                System.out.println("OK");
            } else {
                System.out.println("NG: "+validity.getReferenceMessage(i));
                invalid_counter++;
            }
        }
        if (invalid_counter > 0)
            throw new RuntimeException("It has invalid reference(s).");
    }
}

