/*
 * (C) Copyright IBM Corp. 2000,2001,2002  All rights reserved.
 *
 * US Government Users Restricted Rights Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */

package xss4jsamples.dsig;

import com.ibm.dom.util.StandardErrorHandler;
import com.ibm.xml.dsig.SignatureContext;
import com.ibm.xml.dsig.Validity;
import com.ibm.xml.dsig.XSignature;
import com.ibm.xml.dsig.util.AdHocIDResolver;
import com.ibm.xml.dsig.util.DOMParserNS;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.xml.parsers.DocumentBuilder;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

/**
 *
 * @version $Id: VerifyManifestGUI.java,v 1.4 2004/04/14 06:41:18 kent Exp $
 * @author TAMURA Kent &lt;kent@trl.ibm.co.jp&gt;
 */
public class VerifyManifestGUI {

    private static final String WINDOW_TITLE = "XML-Siganture Manifest Verifier";

    public static void main(String[] argv) {
        InputStream is = null;
        for (int i = 0;  i < argv.length;  i ++) {
            try {
                is = new FileInputStream(argv[i]);
                break;
            } catch (IOException ioe) {
                System.err.println(ioe);
            }
        }
        if (is == null)  is = System.in;

        JFrame frame = new JFrame("Manifest Verifier for W3C/IETF XML-Signature");
        frame.addWindowListener(new WindowAdapter() {
                public void windowClosing(WindowEvent e) {
                    System.exit(0);
                }
            });
        final int FRAMEWIDTH = 480, FRAMEHEIGHT = 320;
        Dimension screen = frame.getToolkit().getScreenSize();
        frame.setBounds((screen.width-FRAMEWIDTH)/2, (screen.height-FRAMEHEIGHT)/2,
                        FRAMEWIDTH, FRAMEHEIGHT);
        Container cont = frame.getContentPane();
        frame.show();

        try {
            //
            // Parses a manifest document.
            //
            DocumentBuilder domp = DOMParserNS.createBuilder();
            domp.setErrorHandler(new StandardErrorHandler());
            Document doc = domp.parse(new InputSource(is));

            //
            // Searches for a Manifest element
            //
            NodeList nodeList = doc.getElementsByTagNameNS(XSignature.XMLDSIG_NAMESPACE,
                                                           "Manifest");
            if (nodeList.getLength() == 0) {
                JOptionPane.showMessageDialog(null,
                                              "The specified document has no Manifest element.",
                                              WINDOW_TITLE, JOptionPane.ERROR_MESSAGE);
                System.exit(1);
            }
            Element manifest = (Element)nodeList.item(0);
            //
            // Checks validity of the Manifest element
            //
            SignatureContext sigContext = new SignatureContext();
            sigContext.setIDResolver(new AdHocIDResolver(doc));
            Validity validity = sigContext.verifyManifest(manifest);

            //
            // Creates result view
            //
            JTable table = new JTable(new VerifyGUI.TableModelImpl(validity));
            table.setFont(new Font("Serif", Font.PLAIN, 20));

            cont.add(new JScrollPane(table));
            cont.validate();
            cont.repaint();

        } catch (SAXException saxe) {
            Exception ex = saxe.getException();
            if (ex == null)  ex = saxe;
            JOptionPane.showMessageDialog(null, ex, WINDOW_TITLE, JOptionPane.ERROR_MESSAGE);
            System.exit(1);
        } catch (Exception ex) {
            JOptionPane.showMessageDialog(null, ex, WINDOW_TITLE, JOptionPane.ERROR_MESSAGE);
            System.exit(1);
        }
    }
}

