/*
 * (C) Copyright IBM Corp. 1999-2001  All rights reserved.
 *
 * US Government Users Restricted Rights Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */

package xss4jsamples.enc;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.security.Key;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.UnrecoverableKeyException;
import java.security.cert.CertificateException;

/**
 * @version $Id: KeyGenerator.java,v 1.6 2003/09/29 06:45:17 imamu Exp $
 * @author Takeshi IMAMURA
 */
public class KeyGenerator {
    //
    // Methods
    //

    public static void main(String[] args) {
	if (args.length < 1) {
	    printUsage();
	    System.exit(1);
	}

        String algorithm = null;
	int size = 0;
	String alias = null, dalias = null;
	char[] keypass = null, dkeypass = null;
	String name = null, dname = null;
	String type = null, dtype = null;
	char[] storepass = null, dstorepass = null;
	String provider = null, dprovider = null;

	String arg = args[0];
	if (arg.equals("-g") && args.length%2 > 0) {
	    for (int i = 1; i < args.length; ) {
		String s = args[i++];
		if (s.equals("-keyalg")) {
		    algorithm = args[i++];

		} else if (s.equals("-keysize")) {
		    size = Integer.parseInt(args[i++]);

		} else if (s.equals("-alias")) {
		    alias = args[i++];

		} else if (s.equals("-keypass")) {
		    keypass = args[i++].toCharArray();

		} else if (s.equals("-keystore")) {
		    name = args[i++];

		} else if (s.equals("-storetype")) {
		    type = args[i++];

		} else if (s.equals("-storepass")) {
		    storepass = args[i++].toCharArray();

		} else if (s.equals("-provider")) {
		    provider = args[i++];

		} else {
		    printUsage();
		    System.exit(1);
		}
	    }
	} else if (arg.equals("-c") && args.length%2 > 0) {
	    for (int i = 1; i < args.length; ) {
		String s = args[i++];
		if (s.equals("-alias")) {
		    alias = args[i++];

		} else if (s.equals("-keypass")) {
		    keypass = args[i++].toCharArray();

		} else if (s.equals("-keystore")) {
		    name = args[i++];

		} else if (s.equals("-storetype")) {
		    type = args[i++];

		} else if (s.equals("-storepass")) {
		    storepass = args[i++].toCharArray();

		} else if (s.equals("-provider")) {
		    provider = args[i++];

		} else if (s.equals("-dalias")) {
		    dalias = args[i++];

		} else if (s.equals("-dkeypass")) {
		    dkeypass = args[i++].toCharArray();

		} else if (s.equals("-dkeystore")) {
		    dname = args[i++];

		} else if (s.equals("-dstoretype")) {
		    dtype = args[i++];

		} else if (s.equals("-dstorepass")) {
		    dstorepass = args[i++].toCharArray();

		} else if (s.equals("-dprovider")) {
		    dprovider = args[i++];

		} else {
		    printUsage();
		    System.exit(1);
		}
	    }
	} else if (arg.equals("-h")) {
	    printUsage();
	    System.exit(0);

	} else {
	    printUsage();
	    System.exit(1);
	}

	if (name == null) {
	    name = System.getProperty("user.home")+File.separator+".keystore";
	}
	if (type == null) {
	    type = KeyStore.getDefaultType();
	}
	if (dname == null) {
	    dname = System.getProperty("user.home")+File.separator+".keystore";
	}
	if (dtype == null) {
	    dtype = KeyStore.getDefaultType();
	}

	try {
	    if (arg.equals("-g")) {
		Key k = generateKey(algorithm, size, provider);
		storeKey(k, alias, keypass, name, type, storepass, provider);

	    } else if (arg.equals("-c")) {
		Key k = getKey(alias, keypass, name, type, storepass, provider);
		storeKey(k, dalias, dkeypass, dname, dtype, dstorepass, dprovider);
	    }
	} catch (Exception e) {
	    e.printStackTrace();
	    System.exit(1);
	}
    }

    private static void printUsage() {
	System.err.println("Usage: java enc.KeyGenerator <option> [<arg>...]");
	System.err.println("where <option> is:");
	System.err.println("   -g -keyalg <algorithm>   [-keysize <size>]");
	System.err.println("      -alias <alias>        -keypass <password>");
	System.err.println("      [-keystore <name>]    [-storetype <type>]");
	System.err.println("      -storepass <password> [-provider <provider>]      generate a key");
	System.err.println("   -c -alias <alias>          -keypass <password>");
	System.err.println("      [-keystore <name>]      [-storetype <type>]");
	System.err.println("      [-storepass <password>] [-provider <provider>]");
	System.err.println("      -dalias <alias>         -dkeypass <password>");
	System.err.println("      [-dkeystore <name>]     [-dstoretype <type>]");
	System.err.println("      -dstorepass <password>  [-dprovider <provider>]   copy a key");
	System.err.println("   -h                                                   print this message");
    }

    private static Key generateKey(String algorithm, int size, String provider)
	throws NoSuchAlgorithmException, NoSuchProviderException {

	if (algorithm == null) {
	    throw new NullPointerException("Algorithm not specified");
	}
	javax.crypto.KeyGenerator kg = null;
	if (provider != null) {
	    kg = javax.crypto.KeyGenerator.getInstance(algorithm, provider);

	} else {
	    kg = javax.crypto.KeyGenerator.getInstance(algorithm);
	}
	if (size > 0) {
	    kg.init(size);
	}

	return kg.generateKey();
    }

    private static Key getKey(String alias, char[] keypass, String name, String type,
			      char[] storepass, String provider)
	throws CertificateException, IOException, KeyStoreException, NoSuchAlgorithmException,
	       NoSuchProviderException, UnrecoverableKeyException {
	KeyStore ks = getKeyStore(name, type, storepass, provider);

	if (alias == null) {
	    throw new NullPointerException("Alias not specified");
	}
	if (keypass == null) {
	    throw new NullPointerException("Key password not specified");
	}

	return ks.getKey(alias, keypass);
    }

    private static KeyStore getKeyStore(String name, String type, char[] password,
					String provider)
	throws CertificateException, IOException, KeyStoreException, NoSuchAlgorithmException,
	       NoSuchProviderException {
	return getKeyStore(new File(name), type, password, provider);
    }

    private static KeyStore getKeyStore(File file, String type, char[] password,
					String provider)
	throws CertificateException, IOException, KeyStoreException, NoSuchAlgorithmException,
	       NoSuchProviderException {
	InputStream is = null;
	if (file.exists()) {
	    is = new FileInputStream(file);
	}

	KeyStore store = null;
	if (provider != null) {
	    store = KeyStore.getInstance(type, provider);

	} else {
	    store = KeyStore.getInstance(type);
	}
	store.load(is, password);

	if (is != null) {
	    is.close();
	}

	return store;
    }

    private static void storeKey(Key key, String alias, char[] keypass, String name,
				 String type, char[] storepass, String provider)
	throws CertificateException, IOException, KeyStoreException, NoSuchAlgorithmException,
	       NoSuchProviderException {
	File f = new File(name);
	KeyStore ks = getKeyStore(f, type, storepass, provider);

	if (alias == null) {
	    throw new NullPointerException("Alias not specified");
	}
	if (keypass == null) {
	    throw new NullPointerException("Key password not specified");
	}
	ks.setKeyEntry(alias, key, keypass, null);

	OutputStream os = new FileOutputStream(f);
	if (storepass == null) {
	    throw new NullPointerException("Keystore password not specified");
	}
	ks.store(os, storepass);
	os.close();
    }
}
