/*
 * (C) Copyright IBM Corp. 1999-2001  All rights reserved.
 *
 * US Government Users Restricted Rights Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */

package xss4jsamples.enc;

import com.ibm.xml.dsig.IDResolver;
import com.ibm.xml.enc.AlgorithmFactoryExtn;
import com.ibm.xml.enc.EncryptedKeyRetriever;
import com.ibm.xml.enc.KeyInfoResolver;
import com.ibm.xml.enc.util.XNIAdHocIdResolver;
import com.ibm.xml.enc.util.XNIKeyStoreKeyInfoResolver;
import com.ibm.xml.enc.util.XNISameDocumentEncryptedKeyRetriever;
import com.ibm.xml.enc.xni.DecryptionParserConfiguration;
import com.ibm.xml.enc.xni.EncryptionParserConfiguration;
import com.ibm.xml.enc.xni.XNIAlgorithmFactoryExtn;

import java.io.IOException;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.cert.CertificateException;
import java.util.Iterator;

import org.apache.xerces.xni.XNIException;
import org.apache.xerces.xni.parser.XMLInputSource;
import org.apache.xerces.xni.parser.XMLParserConfiguration;
import org.w3c.dom.Element;

import xni.Writer;

/**
 * @version $Id: XNICipher.java,v 1.3 2002/12/26 10:49:21 imamu Exp $
 * @author Takeshi IMAMURA
 */
public class XNICipher {
    //
    // Main
    //

    public static void main(String[] args) {
	if (args.length < 1) {
	    printUsage();
	    System.exit(1);
	}

	String arg = args[0];
	String uri = null;
	XMLParserConfiguration config = null;
	if (arg.equals("-e") && args.length > 3) {
	    try {
		XNIAlgorithmFactoryExtn af = new XNIAlgorithmFactoryExtn();
		if (!"*".equals(args[1])) {
		    af.setProvider(args[1]);
		}

		DOMCipher.Keyinfo ki = DOMCipher.getKeyinfo(args[2]);

		uri = args[3];

		KeyInfoResolver kir = getKeyInfoResolver(ki, af, null, null);
		kir.setOperationMode(KeyInfoResolver.ENCRYPT_MODE);

		EncryptionParserConfiguration epc = new EncryptionParserConfiguration();
		for (int i = 4; i < args.length; ) {
		    arg = args[i++];
		    epc.addData(arg);

		    try {
			Element e = DOMCipher.getDocument(args[i++]).getDocumentElement();
			epc.addEncryptedData(arg, e);

		    } catch (Exception e) {
			DOMCipher.printStackTrace(e);
		    }

		    epc.addReplacedData(arg);
		}
		epc.setAlgorithmFactory(af);
		epc.setKeyInfoResolver(kir);

		config = epc;

	    } catch (Exception e) {
		DOMCipher.printStackTrace(e);
		System.exit(1);
	    }
	} else if (arg.equals("-d") && args.length > 3) {
	    try {
		AlgorithmFactoryExtn af = new AlgorithmFactoryExtn();
		if (!"*".equals(args[1])) {
		    af.setProvider(args[1]);
		}

		DOMCipher.Keyinfo ki = DOMCipher.getKeyinfo(args[2]);

		uri = args[3];

		IDResolver ir = new XNIAdHocIdResolver();

		EncryptedKeyRetriever ekr = new XNISameDocumentEncryptedKeyRetriever();

		KeyInfoResolver kir = getKeyInfoResolver(ki, af, ir, ekr);
		kir.setOperationMode(KeyInfoResolver.DECRYPT_MODE);

		DecryptionParserConfiguration dpc = new DecryptionParserConfiguration();
		for (int i = 4; i < args.length; i++) {
		    arg = args[i];
		    dpc.addEncryptedData(arg);
		    dpc.addReplacedEncryptedData(arg);
		}
		dpc.setAlgorithmFactory(af);
		dpc.setKeyInfoResolver(kir);
		dpc.setIdResolver(ir);
		dpc.setEncryptedKeyRetriever(ekr);

		config = dpc;

	    } catch (Exception e) {
		DOMCipher.printStackTrace(e);
		System.exit(1);
	    }
	} else if (arg.equals("-h")) {
	    printUsage();
	    System.exit(0);

	} else {
	    printUsage();
	    System.exit(1);
	}

	Writer writer = new Writer(config);
	try {
	    writer.setOutput(System.out, "UTF-8");
	    writer.parse(new XMLInputSource(null, uri, null));

	} catch (Exception e) {
	    if (e instanceof XNIException) {
		Exception e2 = ((XNIException)e).getException();
		if (e2 != null) {
		    e = e2;
		}
	    }
	    DOMCipher.printStackTrace(e);
	    System.exit(1);
	}
    }

    private static void printUsage() {
	System.err.println("Usage: java enc.XNICipher <option> [<arg>...]");
	System.err.println("where <option> is:");
	System.err.println("   -e <provider> <keyinfo> <target> [<xpath> <template>...]");
	System.err.println("        encrypt elements");
	System.err.println("   -d <provider> <keyinfo> <target> [<xpath>...]");
	System.err.println("        decrypt EncryptedData elements");
	System.err.println("   -h   print this message");
    }

    private static KeyInfoResolver getKeyInfoResolver(DOMCipher.Keyinfo keyinfo,
						      AlgorithmFactoryExtn factory,
						      IDResolver idRes,
						      EncryptedKeyRetriever retriever)
	throws CertificateException, IOException, KeyStoreException, NoSuchAlgorithmException,
	       NoSuchProviderException {
	XNIKeyStoreKeyInfoResolver kiRes
	    = new XNIKeyStoreKeyInfoResolver(DOMCipher.getKeyStore(keyinfo));
	kiRes.setAlgorithmFactory(factory);
	kiRes.setIdResolver(idRes);
	kiRes.setEncryptedKeyRetriever(retriever);
	for (Iterator i = keyinfo.fAlias2Keypass.keySet().iterator(); i.hasNext(); ) {
	    String s = (String)i.next();
	    kiRes.putAliasAndPassword(s, (char[])keyinfo.fAlias2Keypass.get(s));
	}

	return kiRes;
    }
}
