/*
 * (C) Copyright IBM Corp. 1999,2000,2001,2002,2003  All rights reserved.
 *
 * US Government Users Restricted Rights Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * The program is provided "as is" without any warranty express or
 * implied, including the warranty of non-infringement and the implied
 * warranties of merchantibility and fitness for a particular purpose.
 * IBM will not be liable for any damages suffered by you as a result
 * of using the Program. In no event will IBM be liable for any
 * special, indirect or consequential damages or lost profits even if
 * IBM has been advised of the possibility of their occurrence. IBM
 * will not be liable for any third party claims against you.
 */
package xss4jsamples.xacl;

import java.io.StringWriter;
import org.w3c.dom.Document;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.ParserConfigurationException;
import com.ibm.xml.policy.xacl.Hierarchy;
import com.ibm.xml.policy.xacl.HierarchyImpl;
import com.ibm.xml.policy.xacl.XACLProcessor;
import com.ibm.xml.policy.xacl.XACLProcessorImpl;
import com.ibm.xml.policy.xacl.DecisionList;
import com.ibm.xml.policy.xacl.XaclException;
import org.apache.xml.serialize.XMLSerializer;
import org.apache.xml.serialize.OutputFormat;

public class SampleXACLProcessor
{
	public static void main(String[] argv){

		if (argv.length != 4) {
            System.err.println("");
            System.err.println("Usage: java xacl.SampleXACLProcessor  <access_request_file> <target_file> <policy_file> <status_file>");
            System.exit(1);
        }

		String rName, tName, pName, sName;
		Document accessReq=null, target=null, policy=null, status=null;
		rName=argv[0];
		tName=argv[1];
		pName=argv[2];
		sName=argv[3];
		//
		// Build an XML parser
		//
		DocumentBuilder builder=null;
		try{
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
			factory.setNamespaceAware(true);
			builder = factory.newDocumentBuilder();
		}catch(ParserConfigurationException ex){
			System.err.print("Failed to build an XML parser");
			System.err.println(ex.getMessage());
			System.exit(1);
		}
		//
		// Parse the Documents
		//
		System.out.println("loading "+rName);
		try{
			accessReq=builder.parse(rName);
		}catch(Exception ex){
			System.err.print("Failed to parse "+rName);
			System.err.println(ex.getMessage());
			System.exit(1);
		}

		System.out.println("loading "+tName);
		try{
			target=builder.parse(tName);
		}catch(Exception ex){
			System.err.print("Failed to parse "+tName);
			System.err.println(ex.getMessage());
			System.exit(1);
		}

		System.out.println("loading "+pName);
		try{
			policy=builder.parse(pName);
		}catch(Exception ex){
			System.err.print("Failed to parse "+pName);
			System.err.println(ex.getMessage());
			System.exit(1);
		}

		System.out.println("loading "+sName);
		try{
			status=builder.parse(sName);
		}catch(Exception ex){
			System.err.print("Failed to parse "+sName);
			System.err.println(ex.getMessage());
			System.exit(1);
		}

		//Create Hierarchy objects
		Hierarchy gh=null, rh=null;
		Document ghd=null, rhd=null;

		String rhfile=System.getProperty("com.ibm.xml.policy.xacl.rh");
		if(rhfile!=null){
			try{
				rh=new HierarchyImpl(builder.parse(rhfile));
			}catch(Exception ex){
				System.err.print("Failed to parse "+rhfile);
				System.err.println(ex.getMessage());
				System.exit(1);
			}
		}

		String ghfile=System.getProperty("com.ibm.xml.policy.xacl.gh");
		if(ghfile!=null){
			try{
				gh=new HierarchyImpl(builder.parse(ghfile));
			}catch(Exception ex){
				System.err.print("Failed to parse "+ghfile);
				System.err.println(ex.getMessage());
				System.exit(1);
			}
		}
		//
		// Policy Enforcement
		//
		DecisionList decisionList=null;
		Document decision_list=null;
		long start=System.currentTimeMillis();
		try{
			decisionList=XACLProcessorImpl.process(accessReq.getDocumentElement(),
									 target,
									 policy.getDocumentElement(),
									 status.getDocumentElement(),
									 rh,
									 gh);
		}catch(XaclException ex){
			System.err.println("Policy enforcement was not completed.");
			System.err.println(ex.getMessage());
			System.exit(1);
		}
		try{
			decision_list=decisionList.getDocument();
		}catch(ParserConfigurationException ex){
			System.err.print("Failed to build an XML parser");
			System.err.println(ex.getMessage());
			System.exit(1);
		}
		long end=System.currentTimeMillis();
		long time=end-start;
		System.out.println("Policy enforcement was completed.");
		System.out.println("The XACL processing time was "+ time +" ms.");
		//
		// Output
		//
		try{
			if(decision_list!=null){
				OutputFormat format = new OutputFormat();
				format.setIndenting(true);
				format.setPreserveSpace(false);
				StringWriter sw=new StringWriter();
				new  XMLSerializer(sw, format).serialize(decision_list);
				System.out.println(sw.toString());

			}
		}catch(Exception ex){
			System.err.println(ex.getMessage());
			System.exit(1);
		}
	}
}
